/*
 * Silabs SI4711 header file (created from AN305 rev 0.6)
 * Version: $Id: si471x.h 149 2010-02-28 13:24:14Z tipok $
 * 
 */

#ifndef SI471X_H_
#define SI471X_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif


#include <inttypes.h>

/* i2c adresses */
#define		SI4711_I2C_ADDR1		0x63
#define		SI4711_I2C_ADDR2		0x11

/* serial interface commands */
typedef enum commands {
	POWER_UP 		=	0x01,
	GET_REV			=	0x10,
	POWER_DOWN		=	0x11,
	SET_PROPERTY	=	0x12,
	GET_PROPERTY	=	0x13,
	GET_INT_STATUS	=	0x14,
	PATCH_ARGS		=	0x15,
	PATCH_DATA		=	0x16,
	TX_TUNE_FREQ	=	0x30,
	TX_TUNE_POWER	=	0x31,
	TX_TUNE_MEASURE	=	0x32,
	TX_TUNE_STATUS	=	0x33,
	TX_ASQ_STATUS	=	0x34,
	TX_RDS_BUFF		=	0x35,
	TX_RDS_PS		=	0x36,
	GPIO_CTL		=	0x80,
	GPIO_SET		=	0x81
} commands_t;



/* Properties: */
typedef enum properties {
	GPO_IEN					=	0x0001,
	DIGITAL_INPUT_FORMAT	=	0x0101,
	DIGITAL_INPUT_SAMPLE_RATE=	0x0103,
	REFCLK_FREQ				=	0x0201,
	REFCLK_PRESCALE			=	0x0202,
	TX_COMPONENT_ENABLE		=	0x2100,
	TX_AUDIO_DEVIATION		=	0x2101,
	TX_PILOT_DEVIATION		=	0x2102,
	TX_RDS_DEVIATION		=	0x2103,
	TX_LINE_INPUT_LEVEL		=	0x2104,
	TX_LINE_INPUT_MUTE		=	0x2105,
	TX_PREEMPHASIS			=	0x2106,
	TX_PILOT_FREQUENCY		=	0x2107,
	TX_ACOMP_ENABLE			=	0x2200,
	TX_ACOMP_THRESHOLD		=	0x2201,
	TX_ACOMP_ATTACK_TIME	=	0x2202,
	TX_ACOMP_RELEASE_TIME	=	0x2203,
	TX_ACOMP_GAIN			=	0x2204,
	TX_LIMITER_RELEASE_TIME	=	0x2205,
	TX_ASQ_INTERRUPT_SELECT	=	0x2300,
	TX_ASQ_LOW_LEVEL		=	0x2301,
	TX_ASQ_DURATION_LOW		=	0x2302,
	TX_ASQ_HIGH_LEVEL		=	0x2303,
	TX_ASQ_DURATION_HIGH	=	0x2304,                                
	TX_RDS_INTERRUPT_SOURCE	=	0x2C00,
	TX_RDS_PI				=	0x2C01,
	TX_RDS_PS_MIX			=	0x2C02,                                
	TX_RDS_PS_MISC			=	0x2C03,
	TX_RDS_PS_REPEAT_COUNT	=	0x2C04,
	TX_RDS_PS_MESSAGE_COUNT	=	0x2C05,
	TX_RDS_PS_AF			=	0x2C06,
	TX_RDS_FIFO_SIZE		=	0x2C07
} properties_t;



/* GPO_IEN register flags: */
#define	GPO_RDSREP	0x0400
#define	GPO_ASQREP	0x0200
#define	GPO_STCREP	0x0100
#define	GPO_CTSIEN	0x0080
#define	GPO_ERRIEN	0x0040

#define	GPO_RDSIEN	0x0004
#define	GPO_ASQIEN	0x0002
#define	GPO_STCIEN	0x0001

/* TX_COMPONENT_ENABLE arg 1 flags: */
#define	TX_COMPONENT_RDS		0x04
#define	TX_COMPONENT_LMR		0x02
#define	TX_COMPONENT_PILOT		0x01


/* POWER UP ARG1 flags: */
#define		CTSIEN 			0x80
#define		GPO2OEN			0x40
#define		PATCH			0x20
#define		XOSCEN			0x10
#define		FUNC_TX			0x02
#define		FUNC_ID			0x0F



/* POWER UP ARG2 flags: */
typedef enum {
	OPMODE_AN	=	0x50,
	OPMODE_DI	=	0x0F
} asource_t;


/* DIGITAL_INPUT_FORMAT property  flags: */
#define	IFALL	0x0080

enum {
	/* following erratum this val == 0x01,
	 * in AN332 this value == 0x00 whom to believe?*/
	IMODE_I2S		=	0x00,
	IMODE_LEFT_JUSTIFIED	=	0x06,
	IMODE_MSB_1DCLK_RE	=	0x0C,
	IMODE_MSB_2DCLK_RE	=	0x08
};

#define	IMONO	0x0004	/* Mono Audio Mode. */

enum {
	ISIZE_8		=	0x03,
	ISIZE_16	=	0x00,
	ISIZE_20	=	0x01,
	ISIZE_24	=	0x02
}; /* Digital Audio Sample Precision. */




/* TX_LINE_INPUT_MUTE flags: */
typedef enum {
	NOMUTE		=	0x0000,
	RIMUTE		=	0x0001,
	LIMUTE		=	0x0002,
	ALLMUTE		=	0x0003
} mute_t;


/* TX_TUNE_POWER arg 4 flag: */
#define	RFCAP_AUTO		0x00

typedef enum {
	PS_PERCENTAGE_0		=	0x0000,
	PS_PERCENTAGE_12_5	=	0x0001,
	PS_PERCENTAGE_25	=	0x0002,
	PS_PERCENTAGE_50	=	0x0003,
	PS_PERCENTAGE_75	=	0x0004,
	PS_PERCENTAGE_87_5	=	0x0005,
	PS_PERCENTAGE_100	=	0x0006
} ps_percentage;


/* TX_RDS_PS_AF flags: */
#define PS_AF_NONE	0xE0E0 /* NO AF */
#define PS_AF1		0xE100 /* AF1 */


/* TX_RDS_PS_MISC flags: */
#define RDSD3	0x8000
#define RDSD2	0x4000
#define RDSD1	0x2000
#define RDSD0	0x1000
#define FORCEB	0x0800
#define RDSTP	0x0400
#define RDSTA	0x10
#define RDSMS	0x08




/* TX_RDS_INTERRUPT_SOURCE flags: */
#define RDSPSXMIT	0x0010
#define RDSCBUFXMIT	0x0008
#define RDSFIFOXMIT	0x0004
#define RDSCBUFWRAP	0x0002
#define RDSFIFOMT	0x0001




/* TX_RDS_BUFF flags (arg1): */
#define	FIFO		0x80
#define	LDBUFF		0x04
#define MTBUFF		0x02


/* TX_RDS_BUFF
 * TX_ASQ_STATUS
 * TX_TUNE_STATUS
 * */
/* If set this bit clears the interrupt indicator. */
#define INTACK		0x01


/* TX_PREEMPHASIS */
typedef enum {
	FMPE_75		=	0x0000,
	FMPE_50		=	0x0001,
	FMPE_NONE	=	0x0002
} fmpe_t;

/* STATUS flags: */
#define		CTS				0x80
#define		ERR				0x40
#define		RDSINT			0x04
#define		ASQINT			0x02
#define		STCINT			0x01

/* TX_ACOMP_ENABLE parameter flags: */
#define		LIMITEN		0x02
#define		ACEN		0x01


/* TX_ACOMP_RELEASE_TIME */
typedef enum {
	ACOMP_RELEASE_100	=	0x0000,
	ACOMP_RELEASE_200	=	0x0001,
	ACOMP_RELEASE_350	=	0x0002,
	ACOMP_RELEASE_525	=	0x0003,
	ACOMP_RELEASE_1000	=	0x0004
} acomp_release_t;

/* TX_ASQ_INTERRUPT_SELECT flags: */
#define OVERMODIEN	0x0004
#define IALHIEN		0x0002
#define IALLIEN		0x0001




/* input audio params */
typedef struct {
	asource_t	source;
	uint8_t		channels;
	uint16_t	samplerate;
	uint16_t	format;
	uint16_t	bps;
	uint16_t	input_level;
	mute_t		mute;
} ainput_t;

/* radio output params */
typedef struct {
	fmpe_t		preemphasis;
	uint16_t	pilot_tone;
	uint32_t	audio_deviation;
	uint16_t	pilot_deviation;
	uint16_t	rds_deviation;
	uint16_t	output_frequency;
	uint16_t	output_power;
	uint16_t	output_components;
} radio_t;

/* compressor params */
typedef struct {
	int16_t				treshold;
	uint16_t			gain;
	acomp_release_t		release_time;
	uint16_t			attack_time;
} compressor_t;


/* limiter params */
typedef struct {
	uint16_t	level_low;
	uint16_t	level_high;
	uint16_t	duration_low;
	uint16_t	duration_high;
	uint16_t	release_time;
	uint16_t	interrupts;
} limiter_t;

/* rds params */
typedef struct {
	int			send_time;
	uint16_t	interrupts;
	char		*pi_text;
	uint16_t	pi_code;
	ps_percentage	ps_mix;
	uint16_t	ps_misc;
	uint16_t	ps_repeats;
	uint16_t	ps_af;
	uint16_t	fifo_size;	/* need to know for fifo messages
							 * (like timestamp or other)
							 * NOTE: circular buffer will be 
							 * lower on fifo_size blocks */
	char		*rt_text;
} rds_t;


/* all chip params */
typedef struct si471x {
	uint8_t		txoff_onexit;
	uint8_t		started;
	uint8_t		bus;
	uint8_t		chip_id;
	uint8_t		start_mode;
	uint16_t	gp0_interrupts;
	uint32_t	osc_freq;
	ainput_t	*audio_input;
	radio_t		*radio_out;
	uint8_t		compressor_enabled;
	compressor_t	*compressor;
	uint8_t		limiter_enabled;
	limiter_t	*limiter;
	uint8_t		rds_enabled;
	rds_t		*rds;
} si471x_t;


extern si471x_t *transmitter;

si471x_t *si471xDefaults();

int si471xOpen(int8_t bus);
int si471xClose();

int si471xInit(si471x_t *si471x);
int si471xInitCompressor(compressor_t *compressor);
int si471xInitLimiter(limiter_t *limiter);
int si471xInitRadio(radio_t *radio_out);
int si471xInitAudio(ainput_t *audio_input);
int si471xInitRDS(rds_t *rds, uint8_t channels);

int si471xRdsPS(char *psText);
int si471xRdsRadioText(char *message);
int si471xRdsTime();

int si471xShutDown(si471x_t *si471x);

uint8_t si471xInterruptHandle(uint8_t status_in);


#endif /*SI471X_H_*/
